# -*- coding: utf-8 -*-
import boto3
from jinja2 import Environment
from jinja2 import PackageLoader
from jinja2 import select_autoescape
from rich.progress import track

from .index import Index
from .item import Item


class BucketDirGenerator:
    def __init__(self):
        self.template_environment = Environment(
            loader=PackageLoader("bucket_dir", "templates"),
            autoescape=select_autoescape(["html"]),
            trim_blocks=True,
            lstrip_blocks=True,
        )
        self.s3_client = boto3.client("s3")

    def build_indexes(self, contents):
        indexes = {"/": Index("/")}
        for content in contents:
            key_components = content["Key"].split("/")
            item_name = key_components[-1]
            if len(key_components) > 1:
                path_components = key_components[:-1]
                for depth in range(len(path_components)):
                    path = f"/{'/'.join(path_components[:depth + 1])}/"
                    if depth > 0:
                        parent_path = f"/{'/'.join(path_components[:depth])}/"
                    else:
                        parent_path = "/"
                    if path not in indexes:
                        indexes[path] = Index(path)
                    indexes[parent_path].add_item(Item(f"{path_components[depth]}/"))
                full_path = f"/{'/'.join(path_components)}/"
            else:
                full_path = "/"
            if item_name not in ["", "index.html"]:
                indexes[full_path].add_item(
                    Item(name=item_name, modified=content["LastModified"], size=content["Size"])
                )
        return indexes

    def generate(self, bucket, site_name):
        contents = self.get_bucket_contents(bucket)
        indexes = self.build_indexes(contents)
        for path, index in track(indexes.items()):
            index_document = index.render(
                site_name=site_name, template_environment=self.template_environment
            )
            self.upload_index_document_to_s3(bucket, path, index_document)

    def get_bucket_contents(self, bucket):
        paginator = self.s3_client.get_paginator("list_objects_v2")
        page_iterator = paginator.paginate(Bucket=bucket)
        return [content for page in page_iterator for content in page["Contents"]]

    def upload_index_document_to_s3(self, bucket, path, index_document):
        s3_client = boto3.client("s3")
        key = f"{path[1:]}index.html"
        s3_client.put_object(
            Body=index_document.encode(),
            Bucket=bucket,
            CacheControl="max-age=0",
            ContentType="text/html",
            Key=key,
        )



# -*- coding: utf-8 -*-
import sys

import botocore
import click
from rich.console import Console

from .generator import BucketDirGenerator


@click.command()
@click.argument("bucket")
@click.version_option()
def run_cli(bucket):
    """Generate directory listings for an S3 BUCKET.

    BUCKET is the name of the bucket to be indexed."""
    console = Console()
    bucket_dir_generator = BucketDirGenerator()
    try:
        bucket_dir_generator.generate(bucket=bucket, site_name=bucket)
    except botocore.exceptions.NoCredentialsError:
        console.log(
            "Could not discover any AWS credentials. Please supply appropriate credentials."
        )
        sys.exit(1)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['bucket_dir']

package_data = \
{'': ['*'], 'bucket_dir': ['templates/*']}

install_requires = \
['Jinja2>=2.11.3,<3.0.0',
 'boto3>=1.17.11,<2.0.0',
 'click>=7.1.2,<8.0.0',
 'humanize>=3.2.0,<4.0.0',
 'rich>=9.11.0,<10.0.0']

entry_points = \
{'console_scripts': ['bucket-dir = bucket_dir:run_cli']}

setup_kwargs = {
    'name': 'bucket-dir',
    'version': '1.2.0',
    'description': 'Generate directory listings for S3 statically hosted content.',
    'long_description': '# bucket-dir\n\n<a href="https://github.com/hmrc"><img alt="PyPI" src="https://img.shields.io/badge/HMRC-Digital-FFA500?style=flat&labelColor=000000&logo=gov.uk"></a>\n<a href="https://pypi.org/project/bucket-dir/"><img alt="PyPI" src="https://img.shields.io/pypi/v/bucket-dir"></a>\n<img alt="PyPI" src="https://img.shields.io/pypi/pyversions/bucket-dir">\n<a href="https://github.com/hmrc/bucket-dir/blob/master/LICENSE"><img alt="License: Apache 2.0" src="https://img.shields.io/github/license/hmrc/bucket-dir"></a>\n<a href="https://github.com/hmrc/bucket-dir"><img alt="Code style: black" src="https://img.shields.io/badge/code%20style-black-000000.svg"></a>\n\n**bucket-dir** is a utility for generating a browsable directory tree for an AWS S3 bucket.\n\n!["Sample image"](/docs/sample.png "A sample of bucket-dir output.")\n\nIt was built in order to host Maven and Ivy repositories in S3 and serve them via CloudFront, but it could meet other needs too.\n\n## Installation\n\n```\npip install bucket-dir\n```\n## Usage\n\nRun `bucket-dir` with the name of the bucket you wish to index as a parameter:\n\n```\nbucket-dir foo-bucket\n```\n\nUse `bucket-dir --help` for all arguments.\n\nBe sure to provide the command with credentials that allow it to perform ListBucket and PutObject calls against the bucket. E.g. with [aws-vault](https://github.com/99designs/aws-vault):\n\n```\naws-vault exec foo-profile -- bucket-dir foo-bucket\n```\n\n### Character support\n\n`bucket-dir` supports objects using any of the _Safe characters_ listed in the S3 [object key naming guidelines](https://docs.aws.amazon.com/AmazonS3/latest/userguide/object-keys.html#object-key-guidelines).\n\nThe exception to the above rule is using forward slashes consecutively (e.g. `my-folder//my-object`). This results in a folder called `/`, which breaks hyperlinks.\n\nUse of characters in the _Characters that might require special handling_ list is currently unsupported but is theoretically ok.\n\nSome characters in _Characters to avoid_ may also work, but you\'re on your own.\n\n## Development\n\nStart with `make init`. This will install prerequisties and set up a poetry managed virtual environment containing all the required runtime and development dependencies.\n\nUnit testing can be performed with `make test`. If you want to run pytest with other options, use `poetry run pytest ...`.\n\nYou can execute the source code directly with `poetry run bucket-dir`.\n\nFinally, you can build with `make build`. This will update dependencies, run security checks and analysis and finally package the code into a wheel and archive.\n\nPublishing can be performed with `make publish`, but this is only intended to run in CI on commit to the main branch. If running locally, you need to have PyPI credentials set as env vars.\n\nFor other rules, see the [Makefile](Makefile).\n\nIf you are a collaborator, feel free to make changes directly to the main branch. Otherwise, please raise a PR. Don\'t forget to bump the version in [pyproject.toml](pyproject.toml).\n\n## License\n\nThis code is open source software licensed under the [Apache 2.0 License]("http://www.apache.org/licenses/LICENSE-2.0.html").\n',
    'author': 'Dave Randall',
    'author_email': '19395688+daveygit2050@users.noreply.github.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/hmrc/bucket-dir',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
